// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005-2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ObjectGrid.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   01/14/2005   sutter    new code drop
// XD60_M2_B.2  WASX.XD   03/21/2005   sutter    M2 Deliverable
// XD60_M3_B.10 WASX.XD   04/07/2005   sutter    Re-factor datagrid/datagrid.impl
// XD60_M3_B.13 WASX.XD   04/13/2005   jitang    ObjectGrid - JAAS / Security - Drop 1 - Basic Support
// XD60_M3_B.10 WASX.XD   04/21/2005   kjlaw     add getMap method.
// 272560       WASX.XD   05/03/2005   sutter    Resolve Map initialization issue
// 278516       WASX.XD   05/27/2005   kjlaw     update javadoc for reserveSlot method.
// 271885       WASX.XD   06/16/2005   sutter    Remove public methods not implemented for v6
// 279166       WASX.XD   06/16/2005   sutter    Added getEventListeners method
// d327810      WASX.XD   11/30/2005   jitang    Allow passing CredentialGenerator per session
// d332807      WASX.XD   12/14/2005   jitang    take out ObjectGrid.isAuthorizationEnabled
// d338888      WXD601.XD 01/24/2006   kjlaw     javadoc updates.
// d344949      WXD601.XD 02/06/2006   jhanders  BackingMap and ObjectGrid names cannot be null
// bug959       WASX.XD   06/13/2006   dcberg    B.4: Checkpoint replication
// b2692        WASX.XD   09/12/2006   jitang    Refactoring the Stream Query API
// b11208       WASX.XD   03/04/2008   jian.tang Write-behind loader support
// b22228       7.1.1     02/14/2011   kulack    Event Listener remodel in support of byte array plugin enhancements
// b24852       7.1.1     09/15/2011   cdjohnson Add Transaction Isolation configuration to ObjectGrid level.
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid;

import java.io.Serializable;
import java.net.URL;
import java.util.List;

import javax.security.auth.Subject;

import com.ibm.websphere.objectgrid.config.QueryConfig;
//import com.ibm.websphere.objectgrid.continuousquery.ContinuousQueryManager;
import com.ibm.websphere.objectgrid.plugins.EventListener;
import com.ibm.websphere.objectgrid.plugins.ObjectGridEventListener;
import com.ibm.websphere.objectgrid.plugins.ObjectGridLifecycleListener;
import com.ibm.websphere.objectgrid.plugins.ObjectGridPlugin;
import com.ibm.websphere.objectgrid.plugins.TransactionCallback;
import com.ibm.websphere.objectgrid.plugins.TransactionCallbackException;
import com.ibm.websphere.objectgrid.revision.CollisionArbiter;
import com.ibm.websphere.objectgrid.security.SecurityConstants;
import com.ibm.websphere.objectgrid.security.plugins.CredentialGenerator;
import com.ibm.websphere.objectgrid.security.plugins.InvalidSubjectException;
import com.ibm.websphere.objectgrid.security.plugins.ObjectGridAuthorization;
import com.ibm.websphere.objectgrid.security.plugins.SubjectSource;
import com.ibm.websphere.objectgrid.security.plugins.SubjectValidation;

/**
 * This object is used for creating sessions to the ObjectGrid.  It is the
 * central core of the ObjectGrid framework.  Besides creating
 * <code>Session</code>s, it is also responsible for defining <code>BackingMap</code>s,
 * setting a <code>TransactionCallback</code>, 
 * adding event listeners, and managing the security settings.
 *
 * @since WAS XD 6.0, XC10
 */
public interface ObjectGrid {

    /**
     * The default transaction time out value of 10 minutes if no transaction time out value is set.
     *
     * @since WXS 7.1.0.0 FIX1
     */
    final public int DEFAULT_TX_TIMEOUT_VALUE = 10 * 60 ; // 10 minutes;

    /**
     * <p>Gets a Session object that can be used by a single thread at a time.</p>
     *
     * <p>It is not allowed to share this <code>Session</code> object between
     * threads without placing a critical section around it. While the core
     * framework allows the object to move between threads, the
     * <code>TransactionCallback</code> and <code>Loader</code> may prevent this
     * usage, especially in J2EE environments.
     *
     * <p>When the ObjectGrid is a local ObjectGrid, and its security is enabled, this
     * method will use the <code>SubjectSource</code> to get a <code>Subject</code>
     * object and then associate the <code>Subject</code> object with this session .</p>
     *
     * <p>When the ObjectGrid is a distributed ObjectGrid (client server mode), and
     * its security is enabled, this method will utilize the client server security
     * infrastructure to get a secure session. </p>
     *
     * <p>If the <code>initialize()</code> method has not been invoked prior to
     * the first <code>getSession</code> invocation, an implicit initialization
     * will occur.  This ensures that all of the configuration is complete
     * before any runtime usage is required.</p>
     *
     * @return An instance of <code>Session</code>
     *
     * @throws ObjectGridException if an error occurs during processing
     * @throws TransactionCallbackException if the <code>TransactionCallback</code>
     *         throws an exception
     * @throws IllegalStateException if this method is called after the
     *         <code>destroy()</code> method is called.
     *
     * @see #destroy()
     * @see #initialize()
     * @see Session
     * @see SubjectSource
     */
    public Session getSession() throws ObjectGridException, TransactionCallbackException;

    /**
     * Allows the use of a specific <code>Subject</code> rather than use the
     * <code>SubjectSource</code> configured on the ObjectGrid to get a Session.
     * <p>
     * This method should only be used when ObjectGrid security is enabled. If
     * the ObjectGrid security is disabled, the provided <code>Subject</code> object will not be
     * used.
     *
     * <p>If the <code>initialize()</code> method has not been invoked prior to
     * the first <code>getSession</code> invocation, an implicit initialization
     * will occur.  This ensures that all of the configuration is complete
     * before any runtime usage is required.</p>
     *
     * @param subject <code>Subject</code> to associate with the returned Session
     *
     * @return An instance of <code>Session</code>
     *
     * @throws ObjectGridException if an error occurs during processing
     * @throws TransactionCallbackException if the <code>TransactionCallback</code>
     *         throws an exception
     * @throws InvalidSubjectException the subject passed in is invalid based
     *         on the <code>SubjectValidation</code> mechanism.
     * @throws IllegalStateException if this method is called after the
     *         <code>destroy()</code> method is called.
     *
     * @see #destroy()
     * @see #initialize()
     * @see Session
     * @see SubjectValidation
     */
    public Session getSession(Subject subject) throws ObjectGridException, TransactionCallbackException,
            InvalidSubjectException;

    /**
     * Sets the TransactionCallback object.
     * <p>
     * A single cache is a single domain. All <code>Loader</code>s defined for
     * <code>BackingMap</code>s in an ObjectGrid will normally cooperate, thus
     * a corresponding <code>TransactionCallback</code> object needs to be set
     * on the ObjectGrid.
     * <p>
     * A TransactionCallback that implements the {@link ObjectGridLifecycleListener} interface is automatically added
     * as if the {@link #addEventListener(EventListener)} method was called. Any previous
     * callback which implements {@link ObjectGridLifecycleListener} interface is removed as if
     * the {@link #removeEventListener(EventListener)} method was called.
     *
     * <p>
     * A TransactionCallback may implement the {@link ObjectGridPlugin} interface in order
     * to receive enhanced ObjectGrid plug-in lifecycle method calls. The plug-in is
     * also required to correctly implement each of the bean methods related to
     * introspection of its state (for example <code>isInitialized()</code>,
     * <code>isDestroyed()</code>, etc).
     *
     * @param callback An instance of a TransactionCallback
     *
     * @throws IllegalArgumentException if callback is <code>null</code>
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @see #initialize()
     * @see TransactionCallback
     */
    void setTransactionCallback(TransactionCallback callback);

    /**
     * Retrieves the TransactionCallback object.
     * <p>
     * The TransactionCallback can be used in conjunction with the
     * <code>TxID</code> to house transaction-specific context data,
     * such as the connection to the database.
     *
     * @return the argument that was passed to the
     *         <code>setTransactionCallback(TransactionCallback)</code> method of
     *         this interface or a default <code>TransactionCallback</code> object
     *         if setTransactionCallback was not previously called for this ObjectGrid.
     *
     * @see #setTransactionCallback(TransactionCallback)
     * @see TransactionCallback
     */
    TransactionCallback getTransactionCallback();

    /**
     * Sets the CollisionArbiter that is responsible for arbitration of revision conflicts.
     *
     * <p>
     * A CollistionArbiter that implements the {@link ObjectGridLifecycleListener} interface is automatically added
     * as if the {@link #addEventListener(EventListener)} method was called. Any previous
     * arbiter which implements {@link ObjectGridLifecycleListener} interface is removed as if
     * the {@link #removeEventListener(EventListener)} method was called.
     *
     * <p>
     * A CollisionArbiter may implement the {@link ObjectGridPlugin} interface in order
     * to receive enhanced ObjectGrid plug-in lifecycle method calls. The plug-in is
     * also required to correctly implement each of the bean methods related to
     * introspection of its state (for example <code>isInitialized()</code>,
     * <code>isDestroyed()</code>, etc).
     *
     * @param arbiter
     *            The arbitration logic that will be used to resolve collisions.
     * @since 7.1
     */
    void setCollisionArbiter(CollisionArbiter arbiter);

    /**
     * Retrieves the CollisionArbiter that this grid is using to resolve revision collisions.
     *
     * @return The arbitration logic that is responsible for resolving revision collisions.
     * @since 7.1
     */
    CollisionArbiter getCollisionArbiter();

    /**
     * Defines a BackingMap that will be used by the application.
     * <p>
     * Note, to avoid an <code>IllegalStateException</code>, this method must be
     * called prior to the <code>initialize()</code> method.  Also, keep in mind
     * that the <code>getSession</code> methods implicitly call the
     * <code>initialize()</code> method if it has yet to be called by the
     * application.
     *
     * @param name the name of the map being defined.
     *
     * @return a BackingMap reference
     *
     * @throws IllegalArgumentException if name is <code>null</code>
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @see #initialize()
     * @see BackingMap
     */
    BackingMap defineMap(String name);

    /**
     * Creates a BackingMap, but does not associate it with this ObjectGrid.
     * <p>
     * This method is to be used in tandem with the <code>setMaps(List)</code>
     * method, which will associate <code>BackingMap<code>s with this ObjectGrid.
     * These methods are for use when configuring an ObjectGrid with the Spring Framework.
     * <p>
     * Note, to avoid an <code>IllegalStateException</code>, this method must be
     * called prior to the <code>initialize()</code> method.  Also, keep in mind
     * that the <code>getSession</code> methods implicitly call the
     * <code>initialize()</code> method if it has yet to be called by the
     * application.
     *
     * @param name the name of the map being defined.
     * @return a BackingMap reference
     *
     * @throws IllegalArgumentException if name is <code>null</code>
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @see #initialize()
     * @see #setMaps(List)
     * @since WAS XD 6.0.1
     */
    BackingMap createMap(String name);

    /**
     * Clears any <code>BackingMap</code>s that have been previously defined
     * on this ObjectGrid and replaces them with the List of BackingMaps provided.
     * <p>
     * Note, to avoid an <code>IllegalStateException</code>, this method must be
     * called prior to the <code>initialize()</code> method.  Also, keep in mind
     * that the <code>getSession</code> methods implicitly call the
     * <code>initialize()</code> method if it has yet to be called by the
     * application.
     *
     * @param mapList a list of BackingMaps to set on this ObjectGrid.
     *
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @see #createMap(String)
     * @see #initialize()
     * @since WAS XD 6.0.1
     */
    void setMaps(List mapList);

    /**
     * Gets the list of map names currently defined for this ObjectGrid instance.
     * <p>
     * Note, once the <code>initialize()</code> method is called, the
     * <code>List</code> returned will not change.  However, it could
     * change if called prior to initialization.  Also, keep in mind
     * that the <code>getSession</code> methods implicitly call the
     * <code>initialize()</code> method if it has yet to be called by the
     * application.
     *
     * @return a List of String objects, one String per map that was previously
     *         configured by the <code>defineMap(String)</code> or
     *         <code>setMaps(List)</code> method.  An empty List is returned
     *         if no maps are currently defined.
     *
     * @see #defineMap(String)
     * @see #initialize()
     * @see #setMaps(List)
     */
    List /* String */getListOfMapNames();

    /**
     * Returns a BackingMap previously configured by calling the
     * <code>defineMap(String)</code> or <code>setMaps(List)</code> method.
     *
     * @param name the same name that was used as an argument to the
     *        <code>defineMap(String)</code> or <code>createMap(String) method.
     *        A null reference is returned if a map is not associated with this
     *        ObjectGrid for the specified map name.
     *
     * @return the BackingMap instance
     *
     * @see #createMap(String)
     * @see #defineMap(String)
     * @see #setMaps(List)
     */
    BackingMap getMap(String name);

    /**
     * Begins the bootstrapping of the ObjectGrid and Session instances.
     * <p>
     * After this method has been invoked, the configuration of the ObjectGrid is
     * considered complete and is ready for runtime usage.  Any additional
     * configuration method invocations, such as <code>defineMap(String)</code>,
     * will result in an exception.
     *
     * This method is considered optional since the first call to one of the
     * <code>getSession</code> methods will perform an implicit initialization.
     *
     * @throws ObjectGridException if an error occurs during processing
     */
    /* TODO Should initialize() implicitly validate the Session configuration to
     * make sure it's whole?  Or, if that takes too long, maybe we should pass
     * in a flag to initialize to do the validation.
     */
    void initialize() throws ObjectGridException;

    /**
     * Adds an <code>EventListener</code>.
     * <p>
     * Significant events will be communicated to interested listeners through
     * the <code>ObjectGridEventListener<code> and <code>ObjectGridLifecycleListener
     * callback interface.  Multiple event listeners are allowed to be registered, with no
     * implied ordering of event notifications.
     * <p>
     * Note, this method is allowed to be invoked before and after the
     * <code>initialize()</code> method.
     * <p>
     * Object grid plug-ins (<code>TransactionCallback</code>, <code>CollisionArbiter</code>)
     * that implement the <code>ObjectGridLifecycleListener</code> are automatically
     * added as lifecycle listeners when added to the <code>ObjectGrid</code>.
     *
     * @param listener An instance of ObjectGridEventListener or
     *         ObjectGridLifecycleListener
     *
     * @throws IllegalArgumentException if listener is <code>null</code> or not
     *         an instance of ObjectGridEventListener, ObjectGridLifecycleListener.
     * @throws IllegalStateException if this method is called during
     *         initialization by one of the configured plugins and the
     *         ObjectGrid runtime is not in a usable state to initialize
     *         the ObjectGridEventListener.
     *
     * @see ObjectGridEventListener
     * @see ObjectGridLifecycleListener
     * @see EventListener
     */
    void addEventListener(EventListener listener);
    
    /**
     * Provided for compatibility with old releases, use the 
     * {@link #addEventListener(EventListener)} method.
     * 
     * @deprecated This method is deprecated in version 7.1.1, use the {@link #addEventListener(EventListener)} method.
     * 
     * @param listener
     */
    void addEventListener(ObjectGridEventListener listener);

    /**
     * Removes an <code>EventListener</code>.
     * <p>
     * This method removes an <code>ObjectGridEventListener</code>
     * or <code>ObjectGridLifecycleListener</code> that was
     * previously added to this object using the
     * <code>addEventListener(ObjectGridEventListener)</code> or
     * <code>setEventListeners(List)</code> method. If the desired
     * ObjectGridEventListener is not found, no error will be
     * returned.
     * <p>
     * Note, this method is allowed to be invoked before and after the
     * <code>initialize()</code> method.
     *
     * Object grid plug-ins (<code>TransactionCallback</code>, <code>CollisionArbiter</code>)
     * that implement the <code>ObjectGridLifecycleListener</code> are automatically
     * removed as lifecycle listeners when removed from the <code>ObjectGrid</code>.
     *
     * @param listener An instance of ObjectGridEventListener or ObjectGridLifecycleListener
     *
     * @throws IllegalArgumentException if listener is <code>null</code> or not
     *         an instance of <code>ObjectGridEventListener</code>,
     *         <code>ObjectGridLifecycleListener</code>
     *
     * @see #addEventListener(EventListener)
     * @see ObjectGridEventListener
     * @see EventListener
     */
    void removeEventListener(EventListener listener);
    
    /**
     * Provided for compatibility with old releases, use the 
     * {@link #removeEventListener(EventListener)} method.
     * 
     * @deprecated This method is deprecated in version 7.1.1, use the {@link #removeEventListener(EventListener)} method.
     * 
     * @param listener
     */
    void removeEventListener(ObjectGridEventListener listener);

    /**
     * This overwrites the current list of <code>EventListener</code>s
     * and replaces it with the supplied <code>List</code> of <code>EventListener</code>s
     * <p>
     * Note, this method is allowed to be invoked before and after the
     * <code>initialize()</code> method.
     *
     * @param listeners <code>List</code> of <code>ObjectGridEventListener</code>s
     *          and <code>ObjectGridLifecycleListener</code> instances
     *
     * @throws ClassCastException if one of the elements in the provided list
     *         is not an instance of <code>ObjectGridEventListener</code>
     * @throws IllegalArgumentException if listeners is <code>null</code>,
     *         contains a <code>null</code> reference, or contains an instance
     *         of a type other than <code>ObjectGridEventListener</code>
     *          and <code>ObjectGridLifecycleListener</code>
     * @throws IllegalStateException if this method is called during
     *         initialization by one of the configured plugins and the
     *         ObjectGrid runtime is not in a usable state to initialize
     *         the ObjectGridEventListener objects.
     *
     * @deprecated This method is deprecated in version 7.1.1.
     * Use the {@link #addEventListener(EventListener)} or {@link #removeEventListener(EventListener)} methods.
     * Plug-ins that implement the ObjectGridLifecycleListener interface are automatically
     * registered with the grid. Using this method will remove those automatically
     * added listeners.
     *
     * @see EventListener
     * @see #addEventListener(EventListener)
     * @see #removeEventListener(EventListener)
     */
    @Deprecated
    void setEventListeners(List/*<EventListener>*/listeners);

    /**
     * Returns the current list of <code>EventListener</code>s.
     *
     * @return The current list of <code>EventListener</code>s.
     *
     * @see #addEventListener(EventListener)
     * @see EventListener
     * @see ObjectGridEventListener
     * @see ObjectGridLifecycleListener
     */
    List /*<EventListener>*/getEventListeners();

    // methods are deferred to a future release
    // /**
    //  * <p>Instructs the Session to begin quiescing.  In-flight transactions will complete
    //  * while no new transaction can begin.  If the passed in value is true, it will
    //  * abort in-flight transactions with a rollback to
    //  * ensure a timely and orderly shutdown.</p>
    //  *
    //  * <p>Quiesce can be called twice.  First with a false to attempt an orderly
    //  * shutdown.  A second attempt can be called to force the server down.</p>
    //  *
    //  * @param force whether to force the server down. The default value is false/
    //  */
    // // TODO Does quiesce require a timeout value?
    // void quiesce(boolean force);

    /**
     * Gets the name of this ObjectGrid.
     * <p>
     * This method is useful for authorization as all Maps are prefixed with the
     * ObjectGrid name.
     *
     * @return The name of the ObjectGrid.
     *
     * @see #setName(String)
     */
    String getName();

    /**
     * Sets the name of this ObjectGrid.
     *
     * Note, to avoid an <code>IllegalStateException</code>, this method must be
     * called prior to the <code>initialize()</code> method.  Also, keep in mind
     * that the <code>getSession</code> methods implicitly call the
     * <code>initialize()</code> method if it has yet to be called by the
     * application.
     *
     * @param gridName The ObjectGrid name to use.
     *
     * @throws IllegalArgumentException if gridName is <code>null</code>
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     */
    void setName(String gridName);

    /**
     * Allows plugins on this ObjectGrid to reserve slots for use to store
     * context data.
     * <p>
     * Currently the <code>TxID</code> object is the only object that uses
     * slots for storing context data.  TxID slots are used for storing
     * transactional context data.
     * <p>
     * Once a slot is reserved, the slot assignment is permanent and cannot be
     * given back. Note, this method is allowed to be invoked before and after the
     * <code>initialize()</code> method.
     *
     * @param containerName
     *            The name of the Object with the slots. Currently
     *            <code>TxID.SLOT_NAME</code> is the only supported value for
     *            this argument.
     * @return The slot index to use.
     *
     * @throws IllegalArgumentException
     *             if containerName is not <code>TxID.SLOT_NAME</code>.
     *
     * @see TxID#SLOT_NAME
     * @see TxID#getSlot(int)
     * @see TxID#putSlot(int, Object)
     */
    int reserveSlot(String containerName);

    /**
     * Sets the <code>SubjectValidation</code> for this ObjectGrid instance.
     * <p>
     * Passing <code>null</code> to this method removes a previously set
     * <code>SubjectValidation</code> object from an earlier invocation of this method
     * and indicates that this <code>ObjectGrid</code> is not associated with a
     * <code>SubjectValidation</code> object.
     * <p>
     * This method should only be used when ObjectGrid security is enabled. If
     * the ObjectGrid security is disabled, the provided <code>SubjectValidation</code> object
     * will not be used.
     * <p>
     * A <code>SubjectValidation</code> plugin can be used to validate the
     * <code>Subject</code> object passed in is a valid Subject.
     * Please refer to <code>SubjectValidation</code> for more details.
     * <p>
     * Note, to avoid an <code>IllegalStateException</code>, this method must be
     * called prior to the <code>initialize()</code> method.  Also, keep in mind
     * that the <code>getSession</code> methods implicitly call the
     * <code>initialize()</code> method if it has yet to be called by the
     * application.
     *
     * @param subjectValidation the <code>SubjectValidation</code> plugin
     *
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @see #getSession(Subject)
     * @see #initialize()
     * @see SubjectValidation
     */
    void setSubjectValidation(SubjectValidation subjectValidation);

    /**
     * Sets the authorization mechanism.
     * <p>
     * If this method is not invoked, the default authorization mechanism is
     * <code>SecurityConstants.AUTHORIZATION_MECHANISM_JAAS</code>.
     * <p>
     * This method should only be used when ObjectGrid security is enabled. If
     * the ObjectGrid security is disabled, the provide authorization mechanism
     * will not be used.
     * <p>
     * Note, to avoid an <code>IllegalStateException</code>, this method must be
     * called prior to the <code>initialize()</code> method.  Also, keep in mind
     * that the <code>getSession</code> methods implicitly call the
     * <code>initialize()</code> method if it has yet to be called by the
     * application.
     *
     * @param authMechanism the authorization mechanism, must be one of the
     *        final static variable on the <code>SecurityConstants</code> class.
     *
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @see #initialize()
     * @see SecurityConstants#AUTHORIZATION_MECHANISM_CUSTOM
     * @see SecurityConstants#AUTHORIZATION_MECHANISM_JAAS
     */
    void setAuthorizationMechanism(int authMechanism);

    /**
     * Enables the ObjectGrid security.
     * <p>
     * Security on the ObjectGrid level refers to ObjectGrid authorizations.
     * <p>
     * Note, to avoid an <code>IllegalStateException</code>, this method must be
     * called prior to the <code>initialize()</code> method.  Also, keep in mind
     * that the <code>getSession</code> methods implicitly call the
     * <code>initialize()</code> method if it has yet to be called by the
     * application.
     *
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @see #initialize()
     */
    void setSecurityEnabled();

    /**
     * Checks whether security is enabled on this ObjectGrid or not.
     * <p>
     * Security on the ObjectGrid level refers to ObjectGrid authorizations.
     *  Security is disabled by default.
     *
     * @return true if security is enabled on this ObjectGrid; false otherwise.
     *
     * @see #setSecurityEnabled()
     */
    boolean isSecurityEnabled();

    /**
     * Sets the permission check period.
     *
     * <p>This method takes a single parameter indicating how often the customer
     * wants to check the permission used to allow a client access. If the
     * parameter is 0 then every single authorized operation call will
     * ask the authorization mechanism, either JAAS authorization or custom
     * authorization to check if the current Subject has permission. This
     * approach may be prohibitively expensive from a performance point of view
     * depending on the authorization implementation, but if it is required then
     * you can do it. Alternatively, if the parameter is > 0 then it indicates
     * the number of seconds to cache a set of permissions before returning to
     * the authorization mechanism to refresh them. This mechanism provides much
     * better performance, but you run the risk that if the back-end
     * permissions are changed during this time, the ObjectGrid will
     * possibly allow or prevent access even though the back-end security
     * provider has been modified.</p>
     * <p>
     * Note, to avoid an <code>IllegalStateException</code>, this method must be
     * called prior to the <code>initialize()</code> method.  Also, keep in mind
     * that the <code>getSession</code> methods implicitly call the
     * <code>initialize()</code> method if it has yet to be called by the
     * application.
     *
     * @param period the permission check period in seconds.
     *
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @see #initialize()
     */
    void setPermissionCheckPeriod(int period);

    /**
     * Sets the <code>SubjectSource</code> plugin.
     * <p>
     * Passing <code>null</code> to this method removes a previously set
     * <code>SubjectSource</code> object from an earlier invocation of this method
     * and indicates that this <code>ObjectGrid</code> is not associated with a
     * <code>SubjectSource</code> object.
     * <p>
     * A <code>SubjectSource</code> plugin can be used to get a <code>Subject</code>
     * object from the environment to represent the ObjectGrid client.
     * <p>
     * This method should only be used when ObjectGrid security is enabled. If
     * the ObjectGrid security is disabled, the provided <code>SubjectSource</code> object
     * will not be used.
     * <p>
     * Note, to avoid an <code>IllegalStateException</code>, this method must be
     * called prior to the <code>initialize()</code> method.  Also, keep in mind
     * that the <code>getSession</code> methods implicitly call the
     * <code>initialize()</code> method if it has yet to be called by the
     * application.
     *
     * @param source the SubjectSource plugin
     *
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @see #initialize()
     * @see SubjectSource
     */
    void setSubjectSource(SubjectSource source);

    /**
     * Sets the transaction timeout value to a specified number of seconds.
     * <p>
     * Any transaction that is started by use of a Session returned by one of
     * the <code>getSession<code> methods on this interface must complete
     * within the number of seconds specified by the transaction timeout
     * parameter of this method. The timeout value is the maximum number of
     * seconds the transaction is allowed to execute.  If a transaction
     * executes longer than this amount, a <code>TransactionTimeoutException</code>
     * is thrown and the transaction is rolled back even if commit is requested.
     * <p>
     * Note, to avoid an <code>IllegalStateException</code>, this method must be
     * called prior to the <code>initialize()</code> method.  Also, keep in mind
     * that the <code>getSession</code> methods implicitly call the
     * <code>initialize()</code> method if it has yet to be called by the
     * application.
     * <p>
     * The transaction timeout is used by any transaction started by a Session
     * that is returned by the getSession methods of this interface. Since this
     * method must be called prior to getSession method to avoid IllegalStateException,
     * this method only affects transactions that are started after this method is called.
     * If this method is never called, the transaction is allowed unlimited amount of
     * time to complete.
     *
     * @param timeout is the transaction timeout value in seconds. Use a value of 0
     *        to indicate a transaction is allowed unlimited amount
     *        of time so that no TransactionTimeoutException ever occurs.
     *
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @see #initialize()
     * @see Session#TRANSACTION_NO_TIMEOUT
     * @see Session#setTransactionTimeout(int)
     * @see TransactionTimeoutException
     * @since WAS XD 6.0.1
     */
    void setTxTimeout(int timeout);

    /**
     * Gets transaction timeout setting for this ObjectGrid instance.
     *
     * @return timeout value that was passed to the <code>setTxTimeout(int)</code> method or 0
     *         if setTxTimeout was never called.
     *
     * @see #setTxTimeout(int)
     * @since WAS XD 6.0.1
     */
    int getTxTimeout();

    /**
     * Sets the default transaction isolation level for all sessions created by the ObjectGrid.
     * The constants defined in the <tt>Session</tt>
     * interface are the possible transaction isolation levels.
     * The default is {@link Session#TRANSACTION_REPEATABLE_READ}.
     *
     * @param level one of the following <tt>Session</tt> constants:
     * {@link Session#TRANSACTION_READ_UNCOMMITTED}, {@link Session#TRANSACTION_READ_COMMITTED} or
     * {@link Session#TRANSACTION_REPEATABLE_READ}
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     * @throws IllegalArgumentException if this method is includes and invalid transaction isolation level.
     * @since 7.1.1
     */
    void setTxIsolation(int level);

    /**
     * Retrieves the default transaction isolation level.
     * @return the current transaction isolation level.
     * @see #setTxIsolation(int)
     * @since 7.1.1
     */
    int getTxIsolation();

    /**
     * Destroys this instance.
     * <p>
     * This method should be invoked when the ObjectGrid is no longer being
     * used.  When this method is called, the ObjectGrid can free up any
     * resources it is using.  No new Sessions can be created or used after
     * the <code>destroy()</code> has been invoked.  Any in-flight Sessions
     * will be allowed to continue, if the resources are still available to
     * complete processing.
     */
    void destroy();

    /**
     * Get a session using a <code>CredentialGenerator</code>.
     * <p>
     * This method can only be called by the ObjectGrid client in an ObjectGrid
     * client server environment. If ObjectGrid is used in a local model, that is,
     * within the same JVM with no client or server existing, <code>getSession(Subject)</code>
     * or the <code>SubjectSource</code> plugin should be used to secure the ObjectGrid.
     *
     * <p>If the <code>initialize()</code> method has not been invoked prior to
     * the first <code>getSession</code> invocation, an implicit initialization
     * will occur.  This ensures that all of the configuration is complete
     * before any runtime usage is required.</p>
     *
     * @param credGen A <code>CredentialGenerator</code> for generating a credential
     *                for the session returned.
     *
     * @return An instance of <code>Session</code>
     *
     * @throws ObjectGridException if an error occurs during processing
     * @throws TransactionCallbackException if the <code>TransactionCallback</code>
     *         throws an exception
     * @throws IllegalStateException if this method is called after the
     *         <code>destroy()</code> method is called.
     *
     * @see #destroy()
     * @see #initialize()
     * @see CredentialGenerator
     * @see Session
     * @since WAS XD 6.0.1
     */
    Session getSession(CredentialGenerator credGen) throws ObjectGridException, TransactionCallbackException;

    /**
     * Set the QueryConfig object for this ObjectGrid. A QueryConfig object provides query
     * configurations for executing object queries over the maps in this ObjectGrid.
     *
     * @param queryConfig The QueryConfig to associate with this ObjectGrid instance.
     * @throws IllegalArgumentException if queryConfig is <code>null</code>.
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     * @since WAS XD 6.1
     * @see QueryConfig
     */
    void setQueryConfig(QueryConfig queryConfig);

    /**
     * Registers one ore more entities from an entity XML file.<p>
     *
     * Entity registration is required prior to ObjectGrid initialization to bind
     * an Entity with a BackingMap and any defined indices.<p>
     *
     * This method may be called multiple times.
     *
     * @param entityXML the URL of the entity XML that defines the entities.
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @since WAS XD 6.1
     */
    public void registerEntities(URL entityXML);

    /**
     * Register one or more entities based on the class metadata.<p>
     *
     * Entity registration is required prior to ObjectGrid initialization to bind
     * an Entity with a BackingMap and any defined indices.<p>
     *
     * This method may be called multiple times.
     *
     * @param entities one or more annotated entity classes to register as entities.
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @since WAS XD 6.1
     */
    public void registerEntities(Class[] entities);

    /**
     * Indicates the ObjectGrid is a local ObjectGrid.
     */
    public static final int LOCAL = 0;

    /**
     * Indicates the ObjectGrid is a server-side distributed ObjectGrid.
     */
    public static final int SERVER = 1;

    /**
     * Indicates the ObjectGrid is a client-side distributed ObjectGrid.
     */
    public static final int CLIENT = 2;

    /**
     * Returns the type of ObjectGrid.
     * <p>
     * The return value is equivalent to one of the constants declared on this interface,
     * {@link #LOCAL}, {@link #SERVER}, or {@link #CLIENT}.
     *
     * @return the ObjectGrid type
     *
     * @since WAS XD 6.1
     */
    public int getObjectGridType();

    /**
     * Sets the <code>ObjectGridAuthorization</code> for this ObjectGrid instance.
     * <p>
     * Passing <code>null</code> to this method removes a previously set
     * <code>ObjectGridAuthorization</code> object from an earlier invocation of this method
     * and indicates that this <code>ObjectGrid</code> is not associated with a
     * <code>ObjectGridAuthorization</code> object.
     * <p>
     * This method should only be used when ObjectGrid security is enabled. If
     * the ObjectGrid security is disabled, the provided <code>ObjectGridAuthorization</code> object
     * will not be used.
     * <p>
     * A <code>ObjectGridAuthorization</code> plugin can be used to authorize
     * access to the ObjectGrid and maps. Please refer to <code>ObjectGridAuthorization</code> for more details.
     *
     * <p>
     * Note, to avoid an <code>IllegalStateException</code>, this method must be
     * called prior to the <code>initialize()</code> method.  Also, keep in mind
     * that the <code>getSession</code> methods implicitly call the
     * <code>initialize()</code> method if it has yet to be called by the
     * application.
     *
     * @param ogAuthorization the <code>ObjectGridAuthorization</code> plugin
     *
     * @throws IllegalStateException if this method is called after the
     *         <code>initialize()</code> method is called.
     *
     * @see #initialize()
     * @see ObjectGridAuthorization
     * @since WAS XD 6.1
     */
    void setObjectGridAuthorization(ObjectGridAuthorization ogAuthorization);

    /**
     * Set the "access by creator only" mode.
     * <p>
     * Enabling "access by creator only" mode ensures that only the user (represented
     * by the Principals associated with it), who inserts the record into the map,
     * can access (read, update, invalidate, and remove) the record.
     * <p>
     * The "access by creator only" mode can be disabled, or can complement the
     * ObjectGrid authorization model, or it can supersede the ObjectGrid
     * authorization model. The default value is disabled:
     * {@link SecurityConstants#ACCESS_BY_CREATOR_ONLY_DISABLED}.
     * <p>
     * @see SecurityConstants#ACCESS_BY_CREATOR_ONLY_DISABLED
     * @see SecurityConstants#ACCESS_BY_CREATOR_ONLY_COMPLEMENT
     * @see SecurityConstants#ACCESS_BY_CREATOR_ONLY_SUPERSEDE
     *
     * @param accessByCreatorOnlyMode the access by creator mode.
     *
     * @since WAS XD 6.1 FIX3
     */
    void setAccessByCreatorOnlyMode(int accessByCreatorOnlyMode);

    /**
     * Retrieve the current life cycle state of this ObjectGrid.
     * @return the current state.
     * @since 7.1.1
     */
    public ObjectGridLifecycleListener.State getState();

}
