// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  ObjectGridManager.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// 286802       WASX.XD   06/27/2005   cheng1    Factor out non-public factory methods from ObjectGridManager
// 284483       WASX.XD   06/30/2005   cheng1    Need minumum lifecycle support to cause evictor threads to stop
// XD601_M2_B.x WASX.XD   09/02/2005   rajeshk   Adding support for remote client
// 344949       WXD601.XD 02/06/2006   jhanders  Have disconnect return false if not connected.
// 344949       WXD601.XD 02/10/2006   vandersn  remove getCluster method
// XD61         WASX.XD   03/02/2006   haowang   CatalogServer support and two ways bootstraps
// b966         6.1       05/12/2006   jian.tang B2: ObjectGrid Stream Query Support
// b24011       7.1.1     06/20/2011   jhanders  Add getObjectGrid method that takes an override ObjectGridConfiguration
// D7460        8.6       02/20/2012   koehler   ObjectGridManager.disconnect does not check for a null context
// wi8754       WXS 8.5   03/23/2012   cdjohnson Add CatalogDomainManager support.
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid;

import java.net.URL;
import java.util.List;
import java.util.Map;

import com.ibm.websphere.objectgrid.config.ObjectGridConfigFactory;
import com.ibm.websphere.objectgrid.config.ObjectGridConfiguration;
import com.ibm.websphere.objectgrid.security.config.ClientSecurityConfiguration;

/**
 * ObjectGridManager is responsible for creating or retrieving local ObjectGrid instances and
 * connecting to distributed ObjectGrid servers.  Use the {@link ObjectGridManagerFactory} to
 * retrieve an ObjectGridManager.<p>
 *
 * Use the <code>createObjectGrid</code> methods to create a local, in-memory
 * ObjectGrid instance. The <code>createObjectGrid</code> methods give you the choice of caching the created ObjectGrid instance.
 * If you choose to cache the instance, you cannot create an ObjectGrid with the same name unless you remove the
 * previously created ObjectGrid using the <code>removeObjectGrid(String)</code> method. A cached
 * <code>ObjectGrid</code> instance can later be retrieved using the <code>getObjectGrid(String)</code> method.<p>
 *
 * An example of creating a local in-memory ObjectGrid programmatically:
 * <pre>
 * ObjectGridManager ogMgr = ObjectGridManagerFactory.getObjectGridManager();
 * ObjectGrid grid = ogMgr.createObjectGrid("LocalBookStoreGrid");
 * grid.defineMap("Orders");
 * grid.defineMap("Books");
 * grid.initialize();
 * ...
 * grid.destroy();
 * </pre>
 *
 * An example of creating a local in-memory ObjectGrid using an ObjectGrid descriptor XML file:
 * <pre>
 * ObjectGridManager ogMgr = ObjectGridManagerFactory.getObjectGridManager();
 * URL objectgridXML  = Thread.currentThread().getContextClassLoader().getResource("configs/objectgrid.xml");
 * ObjectGrid grid = ogMgr.createObjectGrid("LocalBookStoreGrid", objectgridXML);
 * grid.initialize();
 * ...
 * ogMgr.destroy();
 * </pre>
 *
 * Use the <code>connect</code> methods to connect to a distributed ObjectGrid.  The connect methods
 * return a ClientClusterContext that can then be passed to one of the <code>getObjectGrid</code> methods, which
 * will in turn retrieve a client ObjectGrid instance.<p>
 *
 * An example to connect to a dynamic, distributed ObjectGrid using a catalog server cluster:
 * <pre>
 * ObjectGridManager ogMgr = ObjectGridManagerFactory.getObjectGridManager();
 * ClientClusterContext ccc = ogMgr.connect("catserver1:2809,catserver2:2809", null, null);
 * ObjectGrid grid = ogMgr.getObjectGrid(ccc, "BookStoreGrid");
 * ...
 * ogMgr.disconnect(ccc);
 * </pre>
 *
 * An example to connect to an embedded ObjectGrid server (a server running in the current process):
 * <pre>
 * ObjectGridManager ogMgr = ObjectGridManagerFactory.getObjectGridManager();
 * ClientClusterContext ccc = ogMgr.connect((ClientSecurityConfiguration) null, (URL) null);
 * ObjectGrid grid = ogMgr.getObjectGrid(ccc, "BookStoreGrid");
 * ...
 * ogMgr.disconnect(ccc);
 * </pre>
 *
 * This interface also allows ObjectGrid trace to be disabled completely for performance improvements especially on a
 * processor with a smaller L2 cache.
 *
 * @since WAS XD 6.0, XC10
 */
public interface ObjectGridManager {

    /**
     * Creates a local, in-memory instance of an <code>ObjectGrid</code>. <p>
     *
     * The created ObjectGrid returned by this method is assigned a unique name is <b>not</b> cached by the
     * ObjectGridManager. Use the <code>ObjectGrid.setName(String)</code> method to change the ObjectGrid name.
     *
     * @return an instance of ObjectGrid with a unique name assigned
     *
     * @throws ObjectGridException
     *             if any error occurs during the ObjectGrid creation
     *
     * @see ObjectGrid
     * @see ObjectGrid#setName(String)
     */
    public ObjectGrid createObjectGrid() throws ObjectGridException;

    /**
     * Creates a local, in-memory instance of an ObjectGrid with the specified name.
     * <p>
     * The instance of ObjectGrid created can optionally be cached. If an ObjectGrid with the specified name was
     * previously created and cached, an ObjectGridException will be thrown.
     *
     * @param objectGridName
     *            the name of the ObjectGrid to be created. Must not be <code>null</code>.
     * @param cacheInstance
     *            true, if the ObjectGrid instance should be cached
     *
     * @return an ObjectGrid instance with the specified name.
     *
     * @throws IllegalArgumentException
     *             if objectGridName is <code>null</code>
     * @throws ObjectGridException
     *             if an ObjectGrid with this name has already been cached or any error occurs during the ObjectGrid
     *             creation.
     *
     * @see ObjectGrid
     */
    public ObjectGrid createObjectGrid(String objectGridName, boolean cacheInstance) throws ObjectGridException;

    /**
     * Creates and caches a local, in-memory ObjectGrid instance with the specified name. <p>
     *
     * The ObjectGrid instance created by this method will be cached. Invoking this method is equivalent to invoke
     * <code>createObjectGrid(String, true)</code>
     *
     * @param objectGridName
     *            the Name of the ObjectGrid instance to be created. Must not be <code>null</code>.
     *
     * @return an ObjectGrid instance with the specified name
     *
     * @throws IllegalArgumentException
     *             if objectGridName is <code>null</code>
     * @throws ObjectGridException
     *             if an ObjectGrid with this name has already been cached, or any error occurs during the ObjectGrid
     *             creation
     *
     * @see #createObjectGrid(String, boolean)
     * @see ObjectGrid
     */
    public ObjectGrid createObjectGrid(String objectGridName) throws ObjectGridException;

    /**
     * Creates a local, in-memory ObjectGrid instance based on the specified ObjectGrid name and the ObjectGrid XML configuration file. <p>
     *
     * An ObjectGrid instance is created for the ObjectGrid configuration in the XML file corresponding to the specified
     * ObjectGrid name. If the specified ObjectGrid name cannot be found in the XML file, an exception will be thrown.<p>
     *
     * This returned ObjectGrid instance optionally can be cached.<p>
     *
     * If the URL is null, it will be simply ignored. In this case, this method behaves the same as the
     * <code>createObjectGrid(String, boolean)</code>.
     *
     * @param objectGridName
     *            the Name of the ObjectGrid instance to be returned. Must not be <code>null</code>.
     * @param xmlFile
     *            a URL to a well formed xml file based on the ObjectGrid schema.
     * @param enableXmlValidation
     *            if true the XML is validated
     * @param cacheInstance
     *            a boolean value indicating whether the returned ObjectGrid instance defined in the XML will be cached
     *            or not. If true, the instance will be cached.
     *
     * @return an ObjectGrid instance
     *
     * @throws IllegalArgumentException
     *             if objectGridName is <code>null</code>
     * @throws ObjectGridException
     *             if an ObjectGrid with the same name has been previously cached, an ObjectGrid configuration with the
     *             specified name can be found in the xml file, or any other error occurs during ObjectGrid creation.
     *
     * @see #createObjectGrid(String, boolean)
     * @see ObjectGrid
     * @deprecated Deprecated in version 8.6.  XML validation is always enabled.  Use {@link #createObjectGrid(String, URL, boolean) instead.}
     */
    @Deprecated 
    public ObjectGrid createObjectGrid(String objectGridName, final URL xmlFile, final boolean enableXmlValidation,
            boolean cacheInstance) throws ObjectGridException;

    /**
     * Creates a local, in-memory ObjectGrid instance based on the specified ObjectGrid name and the ObjectGrid XML configuration file. <p>
     *
     * An ObjectGrid instance is created for the ObjectGrid configuration in the XML file corresponding to the specified
     * ObjectGrid name. If the specified ObjectGrid name cannot be found in the XML file, an exception will be thrown.<p>
     *
     * This returned ObjectGrid instance optionally can be cached.<p>
     *
     * If the URL is null, it will be simply ignored. In this case, this method behaves the same as the
     * <code>createObjectGrid(String, boolean)</code>.
     *
     * @param objectGridName
     *            the Name of the ObjectGrid instance to be returned. Must not be <code>null</code>.
     * @param xmlFile
     *            a URL to a well formed xml file based on the ObjectGrid schema.
     * @param cacheInstance
     *            a boolean value indicating whether the returned ObjectGrid instance defined in the XML will be cached
     *            or not. If true, the instance will be cached.
     *
     * @return an ObjectGrid instance
     *
     * @throws IllegalArgumentException
     *             if objectGridName is <code>null</code>
     * @throws ObjectGridException
     *             if an ObjectGrid with the same name has been previously cached, an ObjectGrid configuration with the
     *             specified name can be found in the xml file, or any other error occurs during ObjectGrid creation.
     *
     * @see #createObjectGrid(String, boolean)
     * @see ObjectGrid
     * @since 8.6
     */
    public ObjectGrid createObjectGrid(String objectGridName, final URL xmlFile, boolean cacheInstance) throws ObjectGridException;
    
    /**
     * Creates a <code>List</code> of a local, in-memory <code>ObjectGrid</code> instances
     * based upon the ObjectGrid configurations in the specified ObjectGrid descriptor XML file. <p>
     *
     * The returned ObjecGrid instances can be cached. An ObjectGridException will be thrown when attempting to cache a
     * newly created ObjectGrid that has the same name as an ObjectGrid that has already been cached.
     *
     * @param xmlFile
     *            the file that defines an ObjectGrid or multiple ObjectGrids
     * @param cacheInstances
     *            set to true to cache all ObjectGrid instances created based on the file
     *
     * @return a list of ObjectGrid instances
     *
     * @throws ObjectGridException
     *             if attempting to create and cache an ObjectGrid with the same name as an ObjectGrid that has already
     *             been cached, or any other error occurs during ObjectGrid creation
     *
     * @see ObjectGrid
     * @deprecated Deprecated in version 8.6.  XML validation is always enabled.  Use {@link #createObjectGrids(URL, boolean)} instead.
     */
    public List createObjectGrids(final URL xmlFile, final boolean enableXmlValidation, boolean cacheInstances)
            throws ObjectGridException;
    
    /**
     * Creates a <code>List</code> of a local, in-memory <code>ObjectGrid</code> instances
     * based upon the ObjectGrid configurations in the specified ObjectGrid descriptor XML file. <p>
     *
     * The returned ObjecGrid instances can be cached. An ObjectGridException will be thrown when attempting to cache a
     * newly created ObjectGrid that has the same name as an ObjectGrid that has already been cached.
     *
     * @param xmlFile
     *            the file that defines an ObjectGrid or multiple ObjectGrids
     * @param cacheInstances
     *            set to true to cache all ObjectGrid instances created based on the file
     *
     * @return a list of ObjectGrid instances
     *
     * @throws ObjectGridException
     *             if attempting to create and cache an ObjectGrid with the same name as an ObjectGrid that has already
     *             been cached, or any other error occurs during ObjectGrid creation
     *
     * @see ObjectGrid
     * @since 8.6
     */
    public List createObjectGrids(final URL xmlFile, boolean cacheInstances) throws ObjectGridException;

    /**
     * Creates a <code>List</code> of a local, in-memory <code>ObjectGrid</code> instances
     * based upon the ObjectGrid configurations in the specified ObjectGrid descriptor XML file. <p>
     *
     * The XML file will be validated against the schema and each ObjectGrid instance that is created will be cached. An
     * ObjectGridException will be thrown when attempting to cache a newly created ObjectGrid that has the same name as
     * an ObjectGrid that has already been cached. Using this method is equivalent to calling the
     * <code>createObjectGrids(URL, true, true)</code> method.
     *
     * @param xmlFile
     *            The XML file to process. ObjectGrid(s) will be created based on the configurations what is in the
     *            file.
     *
     * @return A list of ObjectGrid instances that have been created.
     *
     * @throws ObjectGridException
     *             if attempting to create and cache an ObjectGrid with the same name as an ObjectGrid that has already
     *             been cached, or any other error occurs during ObjectGrid creation
     *
     * @see #createObjectGrids(URL, boolean, boolean)
     * @see ObjectGrid
     */
    public List createObjectGrids(final URL xmlFile) throws ObjectGridException;

    /**
     * Creates a local, in-memory ObjectGrid instance based on the specified ObjectGrid name and
     * the ObjectGrid XML configuration file.<p>
     *
     * If there is no ObjectGrid with this name defined in the XML file, an ObjectGridException will be thrown. The XML
     * file will be validated against the schema and the ObjectGrid instance created will be cached. Using this method
     * is equivalent to calling the <code>createObjectGrid(String, URL, true, true)</code> method.
     *
     * @param objectGridName
     *            name of the ObjectGrid to create. This ObjectGrid should be defined in the XML file. Must not be
     *            <code>null</code>.
     * @param xmlFile
     *            the XML file to process
     *
     * @return A newly created ObjectGrid
     *
     * @throws IllegalArgumentException
     *             if objectGridName is <code>null</code>
     * @throws ObjectGridException
     *             if an ObjectGrid with the same name has been previously cached, an ObjectGrid configuration with the
     *             specified name can be found in the xml file, or any other error occurs during ObjectGrid creation.
     *
     * @see #createObjectGrid(String, URL, boolean, boolean)
     * @see ObjectGrid
     */
    public ObjectGrid createObjectGrid(String objectGridName, URL xmlFile) throws ObjectGridException;

    /**
     * Removes a local, in-memory ObjectGrid from the cache of ObjectGrid instances. <p>
     *
     * Invoking this method is equivalent to calling <code>removeObjectGrid(String, false)</code>
     *
     * @param objectGridName
     *            the name of the ObjectGrid instance to remove from the cache
     *
     * @throws ObjectGridException
     *             if an ObjectGrid with the objectGridName was not found in the cache
     *
     * @see #removeObjectGrid(String, boolean)
     */
    public void removeObjectGrid(String objectGridName) throws ObjectGridException;

    /**
     * Removes a local, in-memory ObjectGrid from the cache of ObjectGrid instances
     * and optionally destroys its associated resources.
     *
     * @param objectGridName
     *            the name of the ObjectGrid instance to remove from the cache
     * @param destroy
     *            if true, destroy the objectgrid instance and its associated resources
     *
     * @throws ObjectGridException
     *             if an ObjectGrid with the objectGridName was not found in the cache
     *
     * @see ObjectGrid#destroy()
     */
    public void removeObjectGrid(String objectGridName, boolean destroy) throws ObjectGridException;

    /**
     * Gets a <code>List</code> of the local, in-memory <code>ObjectGrid</code> instances that have been previously cached.
     * <p>
     * This method returns <code>null</code> if no ObjectGrid instances have been cached.
     *
     * @return a list of ObjectGrid instances that have been previously cached or <code>null</code> if there are no
     *         cached ObjectGrid instances
     */
    public List getObjectGrids();

    /**
     * Returns a cached local, in-memory <code>ObjectGrid</code> instance by name.
     * <p>
     * This method returns <code>null</code> if no <code>ObjectGrid</code> with the specified name is currently
     * cached.
     *
     * @param objectGridName
     *            the cached objectgrid name.
     *
     * @return a cached ObjectGrid which currently exists.
     */
    public ObjectGrid getObjectGrid(String objectGridName);

    /**
     * Set the trace specification for the current JVM.
     * <p>
     * This operation is a replace operation, not an append operation. The specification should be of the form:
     * <p>
     * <pre> TraceString := &lt;ComponentString&gt;(:&lt;ComponentString&gt;)* ComponentString :=  &lt;ComponentName&gt;=&lt;type>=&lt;state&gt;(,&lt;type&gt;=&lt;state&gt;)*
     * ComponentName := a java String state := [enabled|disabled] type := [all|debug|event|entryExit] </pre>
     * <p>
     * For example, ObjectGrid=all=enabled
     *
     * @param traceSpec
     *            the new trace specification
     */
    public void setTraceSpecification(String traceSpec);

    /**
     * Retrieves the trace specification for the current JVM.
     * @see #setTraceSpecification(String)
     * @since 7.1
     */
    public String getTraceSpecification();

    /**
     * Sets the trace output to go to a file instead of <code>System.out</code>.
     * <p>
     * The supplied file name can be relative to the working directory or a fully-qualified file name.
     *
     * @param traceFileName
     *            Name of trace file
     */
    public void setTraceFileName(String traceFileName);

    /**
     * Enables or disables ObjectGrid trace for the JVM.
     * <p>
     * Disabling trace improves the performance when ObjectGrid runs on processors whose L2 caches are not large enough
     * to contain the trace enabled code paths. If this is set to false, ObjectGrid trace is suppressed even if it is
     * enabled using <code>setTraceSpecification(String)</code>. By default ObjectGrid trace is enabled.
     *
     * @param enabledFlag
     *            true to enable trace
     *
     * @see #setTraceSpecification(String)
     * @since WAS XD 6.0.1
     */
    public void setTraceEnabled(boolean enabledFlag);

    /**
     * Use this method to connect to an embedded ObjectGrid server.  An embedded ObjectGrid server is typically
     * started in an application server process such as IBM WebSphere Application Server. This method allows
     * connecting to the in-memory ObjectGrid server instance without specifying connection information.<p>
     *
     * This method can be used to connect to both dynamic and static ObjectGrid server deployments.<p>
     *
     * @param securityProps
     *            client security configuration. The value can be <code>null</code> if not running in secure mode.
     * @param overRideObjectGridXml
     *            This parameter can be <code>null</code>. If it is not <code>null</code>, the client side
     *            configuration of the ObjectGrid plugins are overridden with the ObjectGrid configuration using this
     *            <code>URL</code>. If this parameter is <code>null</code>, client side ObjectGrid plugins can be
     *            overridden by providing an overrideMap to <code>setOverrideObjectGridConfigurations(Map)</code> or
     *            by calling <code>putOverrideObjectGridConfigurations(String, List)</code>.
     *            <p>
     *            In cases where this parameter is not null, and overriding configuration objects have been provided to
     *            the ObjectGridManager by <code>setOverrideObjectGridConfigurations(Map)</code> or
     *            <code>putOverrideObjectGridConfigurations(String, List)</code>, the configuration based on the XML
     *            file will be used to override ObjectGrid settings. Overriding objects provided to
     *            <code>setOverrideObjectGridConfigurations(Map)</code> or
     *            <code>putOverrideObjectGridConfigurations(String, List)</code> will be ignored.
     *            <p>
     *            Not all plugins can be overridden. For details please see the ObjectGrid documentation.
     *
     * @return a ClientClusterContext representing the cluster ObjectGrid configuration to which the client is
     *         connected.
     *
     * @throws ConnectException
     *             if any error occurs connecting to the server
     *
     * @see ClientClusterContext
     * @see ClientSecurityConfiguration
     * @since WAS XD 6.0.1
     */
    public ClientClusterContext connect(ClientSecurityConfiguration securityProps, URL overRideObjectGridXml)
            throws ConnectException;

    /**
     * Connects a client process to a remote ObjectGrid catalog server or catalog server cluster (a dynamic ObjectGrid
     * deployment topology). The result ClientClusterContext can then
     * be used to get any ObjectGrid reference managed by that catalog server cluster.
     *
     * @param catalogServerEndpoints
     *            A concatenated list of host/port pairs belonging to the catalog servers in the form "host:port<,host:port>".
     *            This list can be arbitrarily long and is used for bootstrapping only. The first viable address will be
     *            used.
     * @param securityProps
     *            This parameter may be null if the client does not wish to establish a secure connection with the
     *            server.
     * @param overRideObjectGridXml
     *            This parameter can be <code>null</code>. If it is not <code>null</code>, the client side
     *            configuration of the ObjectGrid plugins are overridden with the ObjectGrid configuration using this
     *            <code>URL</code>. If this parameter is <code>null</code>, client side ObjectGrid plugins can be
     *            overridden by providing an overrideMap to <code>setOverrideObjectGridConfigurations(Map)</code> or by
     *            calling <code>putOverrideObjectGridConfigurations(String, List)</code>.
     *            <p>
     *            In cases where this parameter is not null, and overriding configuration objects have been provided to
     *            the ObjectGridManager by <code>setOverrideObjectGridConfigurations(Map)</code> or
     *            <code>putOverrideObjectGridConfigurations(String, List)</code>, the configuration based on the XML
     *            file will be used to override ObjectGrid settings. Overriding objects provided to
     *            <code>setOverrideObjectGridConfigurations(Map)</code> or
     *            <code>putOverrideObjectGridConfigurations(String, List)</code> will be ignored.
     *            <p>
     *            Not all plugins can be overridden. For details please see the ObjectGrid documentation.
     * @return a ClientClusterContext representing the cluster ObjectGrid configuration to which the client is
     *         connected.
     * @throws ConnectException
     *             If there is a problem connecting to the addresses given.
     */
    public ClientClusterContext connect(String catalogServerEndpoints, ClientSecurityConfiguration securityProps,
            URL overRideObjectGridXml) throws ConnectException;

    /**
     * Disconnects a client process from an ObjectGrid server
     *
     * @param context
     *            the <code>ClientClusterContext</code> object returned from a previous call to one of the
     *            <code>connect</code> methods The caller must guarantee this parameter is not <code>null</code>.
     *
     * @return <code>true</code> if the disconnect was successful, <code>false</code> if the supplied context was
     *         not connected
     *         
     * @throws IllegalArgumentException if the ClientClusterContext is null        
     *
     * @see ClientClusterContext
     * @since WAS XD 6.0.1
     */
    public boolean disconnect(ClientClusterContext context);

    /**
     * Returns a client <code>ObjectGrid</code> instance corresponding to an <code>ObjectGrid</code> in an
     * ObjectGrid cluster.
     * <p>
     * This method is equivalent to calling <code>getObjectGrid(context, objectGridName, null)</code>
     *
     * @param context
     *            the <code>ClientClusterContext</code> object returned from a previous call to one of the
     *            <code>connect</code> methods The caller must guarantee this parameter is not <code>null</code>.
     * @param objectGridName
     *            the name of the requested client ObjectGrid
     *
     * @return a client ObjectGrid instance corresponding to a remote ObjectGrid
     *
     * @throws IllegalArgumentException if either provided parameter is null
     * @throws ObjectGridRuntimeException is the <code>ObjectGrid</code> with the specified name is not hosted in
     *         any eXtreme Scale servers managed by the catalog server
     *
     * @see ClientClusterContext
     * @see ObjectGrid
     * @see #getObjectGrid(ClientClusterContext, String, ObjectGridConfiguration)
     * @since WAS XD 6.0.1
     */
    public ObjectGrid getObjectGrid(ClientClusterContext context, String objectGridName);// @XD601_M2_B.x

    /**
     * Returns a client <code>ObjectGrid</code> instance corresponding to an <code>ObjectGrid</code> in an
     * ObjectGrid cluster.
     * <p>
     * This method replaces the get/set/putOverrideObjectGridConfigurations methods. Those methods had thread safety
     * issues. In addition they were global in nature so we end up having configuration override happen for all client
     * connections unless it was managed correctly. If ClientClusterContext was used previously to get an ObjectGrid for
     * the given name, the same ObjectGrid instance is returned even if the <code>overrideOGConfig</code> parameter is
     * different.
     * 
     * @param context
     *            the <code>ClientClusterContext</code> object returned from a previous call to one of the
     *            <code>connect</code> methods The caller must guarantee this parameter is not <code>null</code>.
     * @param objectGridName
     *            the name of the requested client ObjectGrid
     * 
     * @param overrideOGConfig
     *            This parameter can be <code>null</code>. If it is not <code>null</code>, the client side
     *            configuration of the ObjectGrid plugins are overridden with the <code>ObjectGridConfiguration</code>
     *            provided. The provided override configuration takes precedence over any other provided override
     *            configuration for the requested ObjectGrid name provided by the <code>connect</code>,
     *            <code>putOverrideObjectGridConfigurations</code>, and
     *            <code>putOverrideObjectGridConfigurations</code> methods.
     *            <p>
     *            Not all plugins can be overridden. For details please see the ObjectGrid documentation.
     * @return a client ObjectGrid instance corresponding to a remote ObjectGrid
     * 
     * @see ObjectGridConfiguration
     * @see ObjectGridConfigFactory
     * @since 7.1.1
     */
    public ObjectGrid getObjectGrid(ClientClusterContext context, String objectGridName, ObjectGridConfiguration overrideOGConfig);

    /**
     * Override ObjectGrid settings on client side ObjectGrids by passing in a Map where each key corresponds to a
     * cluster name or domain name and each value is a <code>List</code> of <code>ObjectGridConfiguration</code> objects to be
     * overridden.
     * <p>
     * Client side configuration of ObjectGrid and BackingMap plugins are overridden using the ObjectGridConfiguration
     * values provided in the List. To override a Plugin, each <code>ObjectGridConfiguration</code> object must have a
     * name that matches the name of the ObjectGrid to be overridden. <code>BackingMapConfiguration</code> objects
     * must have the same name as a BackingMap and be associated with the proper<code>ObjectGridConfiguration</code>.
     * <p>
     * Not all plugins can be overridden. ObjectGrid plugins that can be overridden on the client side are
     * TransactionCallback and ObjectGridEventListener. BackingMap plugins that can be overridden on the client side are
     * Evictor and MapEventListener. Settings for the builtin Evictor can also be altered on the BackingMap. These
     * settings include numberOfBuckets, timeToLive, and ttlEvictorType.
     *
     * @param overrideMap
     *            a Map that will be used to override ObjectGrid settings on the client side. To override client side
     *            settings, each key of the Map must be a String with a value that corresponds to a cluster name or domain name. Each
     *            value of the overrideMap must be a <code>java.util.List</code>. The List elements must be
     *            <code>ObjectGridConfiguration</code> objects. Each call to a connect method with a clusterName that
     *            matches a key in the overrideMap will result in the client side settings being overridden using the
     *            List of ObjectGridConfiguration objects provided in the key's corresponding value. Pass in
     *            <code>null</code> to clear an overrideMap that was previously set and thereby remove any overriding
     *            settings from future connect calls.
     *
     * @throws IllegalArgumentException
     *            if any keys or values are null or if keys or values are of the wrong type
     *
     * @since WAS XD 6.0.1.2
     * @see #connect(String, ClientSecurityConfiguration, URL)
     * @see #connect(ClientSecurityConfiguration, URL)
     * @see com.ibm.websphere.objectgrid.config.ObjectGridConfiguration
     * @see com.ibm.websphere.objectgrid.config.BackingMapConfiguration
     * @deprecated This method is replaced in 7.1.1 with the getObjectGrid method that takes a ObjectGridConfigurations.
     * @see #getObjectGrid(ClientClusterContext, String, ObjectGridConfiguration)
     */
    public void setOverrideObjectGridConfigurations(Map overrideMap);

    /**
     * Put an entry into the Map that is used to override client side ObjectGrid and BackingMap plugins.
     *
     * @param clusterName
     *            to be used as a key in the Map used to override client side ObjectGrid plugins. If a connect method is
     *            called with a matching clusterName, the client side ObjectGrid and BackingMap plugins can be
     *            overridden using the objectGridConfigList.
     *
     *            In the dynamic environment, use the domain name to override ObjectGrid settings.
     * @param objectGridConfigList
     *            a List of <code>ObjectGridConfiguration</code> objects that will be used to override client side
     *            ObjectGrid settings if a connect method is called with a cluster name that matches the clusterName on
     *            this method
     *
     * @throws IllegalArgumentException
     *            if the clusterName or objectGridConfigList is null
     * @deprecated This method is replaced in 7.1.1 with the getObjectGrid method that takes a ObjectGridConfigurations.
     * @see #getObjectGrid(ClientClusterContext, String, ObjectGridConfiguration)
     */
    public void putOverrideObjectGridConfigurations(String clusterName, List objectGridConfigList);

    /**
     * Get the Map that is used to override client side ObjectGrid and BackingMap plugins.
     *
     * @return the Map that was set by the call to <code>setOverrideObjectGridConfigurations</code>. The Map may also
     *         have entries that were put there using the <code>putOverrideObjectGridConfigurations</code> method.
     * @deprecated This method is replaced in 7.1.1 with the getObjectGrid method that takes a ObjectGridConfigurations.
     * @see #getObjectGrid(ClientClusterContext, String, ObjectGridConfiguration)
     */
    public Map getOverrideObjectGridConfigurations();
    
    /**
     * Retrieve the CatalogDomainManager for this ObjectGridManager.
     * @return the CatalogDomainManager, if available.  Returns null if a CatalogDomainManager is not supported
     * in the current runtime environment.
     * @since 8.5
     */
    public CatalogDomainManager getCatalogDomainManager();
}
