// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.4 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/TTLType.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:25:48
//
// Module  :  TTLType.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
//              WASX.XD   Jun 21, 2005 kjlaw     new code drop
// b896                   05/01/2006   vandersn  serial with configuration
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid;

import java.io.Serializable;

/**
 * Every BackingMap in ObjectGrid has a built in timed based evictor
 * that is referred to as "time to live" evictor or TTL evictor.  Each BackingMap
 * entry has an expiration time that determines how long the entry is
 * allowed to live in the BackingMap.  When the expiration time is reached,
 * the TTL evictor causes the expired entry to be evicted from
 * the BackingMap.  This class is used to define the TTLType value constants
 * that determine how the the expiration time is computed for a map entry.
 *
 * @since WAS XD 6.0, XC10
 *
 * @see BackingMap#setTtlEvictorType(TTLType)
 */
public class TTLType implements Serializable
{
    private static final long serialVersionUID = -2115196396369253602L;

    /**
     * A TTLType.NONE indicates an entry has no expiration time and
     * is allowed to live in the BackingMap until the application explicitly
     * removes or invalidates the entry or a user defined evictor
     * evicts it.
     */
    public final static TTLType NONE = new TTLType((byte) 0);

    /**
     * A TTLType.CREATION_TIME indicates an entry expiration time is the sum
     * of the creation time of the entry plus the "time to live" value. The
     * "time to live" value is set using the <code>BackingMap.setTimeToLive(int)</code>
     * method and is the same for every entry and can <b>not</b> be
     * changed by the application by using the <code>ObjectMap.setTimeToLive(int)</code> method.
     * It can only be set prior to ObjectGrid initialization
     * by use of the <code>BackingMap.setTimeToLive(int)</code> method.
     *
     * @see BackingMap#setTimeToLive(int)
     */
    public final static TTLType CREATION_TIME = new TTLType((byte) 1);

    /**
     * A TTLType.LAST_ACCESS_TIME indicates an entry expiration time is the sum
     * of the last access time of the entry plus the "time to live" value.  By
     * default, the time to live value is set using the
     * <code>BackingMap.setTimeToLive(int)</code> method and the default can be overridden
     * by the application by using the <code>ObjectMap.setTimeToLive(int)</code> method.
     *
     * @see BackingMap#setTimeToLive(int)
     * @see ObjectMap#setTimeToLive(int)
     */
    public final static TTLType LAST_ACCESS_TIME = new TTLType((byte) 2);

    /**
     * A TTLType.LAST_UPDATE_TIME indicates an entry expiration time is the sum
     * of the last update time of the entry plus the "time to live" value.  By
     * default, the time to live value is set using the
     * <code>BackingMap.setTimeToLive(int)</code> method and the default can be overridden
     * by the application by using the <code>ObjectMap.setTimeToLive(int)</code> method.
     * The difference between this TTLType and LAST_ACCESS_TIME is that fetch operations do not
     * cause the entry expiration time to be updated.
     *
     * @see BackingMap#setTimeToLive(int)
     * @see ObjectMap#setTimeToLive(int)
     * @since 7.1
     */
    public final static TTLType LAST_UPDATE_TIME = new TTLType((byte) 3);
    
    /**
     * Given the raw value of a TTLType, this method returns a TTLType object, or null if the raw
     * value does not match an existing type.  This method is used to deserialize this object.
     * 
     * @param id the raw value of a TTLType
     * @return the TTLType corresponding to the raw input value
     * @since 8.6
     */
    public final static TTLType valueOf(byte id) {
        switch(id) {
	        case 0: return NONE;
	        case 1: return CREATION_TIME;
	        case 2: return LAST_ACCESS_TIME;
	        case 3: return LAST_UPDATE_TIME;
	        default: return null;
        }
    }

    // Id that uniquely identifies the TTLType instance.
    final private byte ivUniqueId;

    /**
     * Hidden CTOR for enumerated type idiom to use.
     *
     * @param uniqueId is the id that is unique to the constructed TTLType instance.
     */
    private TTLType( byte uniqueId)
    {
        ivUniqueId = uniqueId;
    }

    /**
     * Hidden default CTOR for enumerated type idiom to use.
     */
    private TTLType()
    {
        this((byte) 0);
    }
    
    /**
     * Get the raw value of this TTLType.  This method is used to serialize this object.
     * @return the raw value of this TTLType.
     * @since 8.6
     */
    public byte getId() {
    	return ivUniqueId;
    }

    private Object readResolve () throws java.io.ObjectStreamException {
        switch(ivUniqueId){
            case 0: return NONE;
            case 1: return CREATION_TIME;
            case 2: return LAST_ACCESS_TIME;
            case 3: return LAST_UPDATE_TIME;
            default: return null;
        }
    }

    /**
     * Returns a string representation of the <code>TTLType</code>.
     *
     * @return  a string representation of the <code>TTLType</code>.
     */
    public String toString()
    {
        switch (ivUniqueId)
        {
        case 0: return "NONE TTLType";              //$NON-NLS-1$
        case 1: return "CREATION_TIME TTLType";     //$NON-NLS-1$
        case 2: return "LAST_ACCESS_TIME TTLType";  //$NON-NLS-1$
        case 3: return "LAST_UPDATE_TIME TTLType";  //$NON-NLS-1$
        default: return "UNKNOWN TTLType";          //$NON-NLS-1$
        }
    }
}
