// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ObjectGridEventGroup.java
//
// Source File Description:
//
//    See javadoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins;

import java.util.Collection;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.Session;

/**
 * This is a set of single method interfaces for fine grained
 * events delivered for an ObjectGrid. Classes implementing
 * these interfaces AND ObjectGridEventListener can receive
 * these events. If an ObjectGridEventListener implements ANY of
 * these interfaces that only the specific methods on the interfaces
 * implemented will be called.
 *
 * @since WAS XD 6.1, XC10
 *
 * @see ObjectGridEventListener
 */
public interface ObjectGridEventGroup
{
	/**
	 * These events are fired when an ObjectGrid shard is initialized and destroyed. A shard
	 * can be activated/deactivated multiple times within these two events.
	 *
	 */
	static public interface ShardLifecycle
	{
	    /**
	     * Invoked when an <code>ObjectGrid</code> is initialized.
	     * <p>
	     * A usable <code>Session</code> instance is passed to this listener to
	     * provide all of the necessary access to the various ObjectGrid objects.
	     *
	     * @param session a Session instance that this listener is associated with.
	     *
	     * @see ObjectGrid#initialize()
	     */
		void initialize(Session session);
	    /**
	     * <p>Called when the <code>ObjectGrid</code> associated with this listener is
	     * destroyed.</p>
	     *
	     * <p>This method is the opposite of the <code>initialize</code> method. When
	     * it is called, the listener can free up any resources it uses.</p>
	     *
	     * @see ObjectGrid#destroy()
	     */
		void destroy();
	}

	/**
	 * These events are fired when a shard is made a primary shard and when the shard is demoted from a primary.
	 *
	 */
	static public interface ShardEvents
	{
		/**
		 * This is called when a shard is promoted to a primary.
		 * @param grid This is a local reference to the shard containing the primary data.
		 */
		void shardActivated(ObjectGrid grid);
		/**
		 * This is called when a primary shard is demoted to a replica. This can
		 * happen is the balancer decides the primary is better placed in a different
		 * container. Replication is still active until this method returns to the
		 * caller. If any application controlled transactions are in flight then they
		 * should be stopped before returning. Once this method returns then any remaining
		 * transactions will fail.
		 * @param grid A reference to the shard.
		 */
		void shardDeactivate(ObjectGrid grid);
	}

	/**
	 * These events are called every single transaction. These are primarily used when
	 * transaction level listening is required. This is usually for pushing changes
	 * or invalidation events to peer caches for simple scenarios.
	 *
	 */
	static public interface TransactionEvents
	{
		   /**
	     * Signals the beginning of a <code>Session</code> transaction.
	     * <p>
	     * A stringified version of the TxID is provided for correlating with the
	     * end of the transaction, if so desired.  The type of transaction is also
	     * provided by the isWriteThroughEnabled boolean parameter.
	     *
	     * @param txid Stringified version of the TxID
	     * @param isWriteThroughEnabled boolean flag indicating whether the Session
	     *                              transaction was started using the
	     *                              <code>Session.beginNoWriteThrough()</code>.
	     *                              method. <code>false</code> is passed if
	     *                              <code>beginNoWriteThrough()</code> was used.
	     *
	     * @see Session#begin()
	     * @see Session#beginNoWriteThrough()
	     */
	    void transactionBegin(String txid, boolean isWriteThroughEnabled);
	    /**
	     * <p>Signals the ending of a <code>Session</code> transaction.</p>
	     *
	     * <p>A string version of the TxID is provided for correlating with the
	     * begin of the transaction, if so desired. Map changes are also reported
	     * with the collection of <code>LogSequence</code>s passed to this method.
	     * Typical uses of this event are for customers doing custom peer
	     * invalidation or peer commit push. This event listener gives them the
	     * changes. Calls to this method are made after commit and are sequenced so
	     * that they are delivered one by one, not in parallel. The event order is
	     * the commit and rollback order.</p>
	     *
	     * @param txid string version of the TxID
	     * @param isWriteThroughEnabled boolean flag indicating whether the Session
	     *                              transaction was started using the
	     *                              <code>Session.beginNoWriteThrough()</code>.
	     *                              method. <code>false</code> is passed if
	     *                              <code>beginNoWriteThrough()</code> was used.
	     * @param committed a boolean flag indicating whether the transaction was
	     *                  committed (<code>true</code>) or rolled back
	     *                  (<code>false</code>)
	     * @param changes   a Collection of LogSequences representing the changes
	     *                  that were committed or rolled back.
	     *
	     * @see LogSequence#isRollback()
	     * @see Session#begin()
	     * @see Session#beginNoWriteThrough()
	     * @see Session#commit()
	     * @see Session#rollback()
	     */
	    void transactionEnd(String txid, boolean isWriteThroughEnabled, boolean committed, Collection /*<LogSequence>*/ changes);
	}
}
