// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.5 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/TransactionCallback.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:23:30
//
// Module  :  TransactionCallback.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   02/15/2005   sutter    new code drop
// XD60_M3_B.10 WASX.XD   04/07/2005   sutter    Re-factor datagrid/datagrid.impl
// RTC:40000    7.1.1.0   08/24/2011   weaverm   Add BeforeCommit interface
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins;

import java.util.Collection;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.Session;
import com.ibm.websphere.objectgrid.TxID;

/**
 * Calling methods on a <code>Session</code> will send corresponding events
 * to the TransactionCallback. An ObjectGrid can have zero or one
 * TransactionCallback. BackingMaps defined on an ObjectGrid with
 * a TransactionCallback should have corresponding Loaders.
 * <p>
 * A TransactionCallback works with <code>Loader</code>s and place transaction
 * specific objects in slots on the <code>TxID</code> object that
 * <code>Loader</code>s can obtain. Examples are database connections, prepared
 * statement caches, etc. The TransactionCallback should reserve slots in the
 * <code>TxID</code> by calling <code>ObjectGrid.reserveSlot(String)</code>
 * using the name <code>TxID.SLOT_NAME</code>. The TransactionCallback can then
 * put an object at that index in the <code>TxID</code>. A <code>Loader</code>
 * can retrieve the index used by the <code>TransactionCallback</code> by
 * calling an internal method on the <code>TransactionCallback</code>'s
 * implementation.  A reference to the configured <code>TransactionCallback</code>
 * can be found using the
 * <code>TxID.getSession().getObjectGrid().getTransactionCallback()</code>
 * code sequence.
 * <p>
 * A <code>TransactionCallback</code> implementation that also implements the {@link ObjectGridLifecycleListener}
 * interface will be automatically added as an {@link EventListener} on the {@link ObjectGrid} when the
 * callback is set on the object grid.
 * <p>
 * A TransactionCallback may implement the {@link ObjectGridPlugin} interface in order
 * to receive enhanced ObjectGrid plug-in lifecycle method calls. The plug-in is
 * also required to correctly implement each of the bean methods related to
 * introspection of its state (for example <code>isInitialized()</code>,
 * <code>isDestroyed()</code>, etc).
 *
 *
 * @see Loader
 * @see ObjectGrid#addEventListener(EventListener)
 * @see ObjectGrid#getTransactionCallback()
 * @see ObjectGrid#reserveSlot(String)
 * @see ObjectGrid#setTransactionCallback(TransactionCallback)
 * @see Session#getObjectGrid()
 * @see TxID#putSlot(int, Object)
 * @see TxID#getSlot(int)
 * @see TxID#getSession()
 *
 * @since WAS XD 6.0, XC10
 */
/*
 * TODO This callback will not be necessary when the Session is supported as an
 *   XAResource.
 */
public interface TransactionCallback {
    /**
     * The BeforeCommit optional mix-in interface for the TransactionCallback plug-in interface allows plug-ins to be
     * notified at the beginning of a {@link Session#commit()}. Implementations can use the <code>beforeCommit()</code>
     * method to validate changed data in the transaction and modify the data.
     * 
     * @since 7.1.1
     */
    public interface BeforeCommit extends TransactionCallback {

        /**
         * Invoked at the beginning of a <code>Session.commit()</code>.
         * <p>
         * 
         * Use the <code>TransactionContext.getLogSequences()</code> method to retrieve the changes made by this
         * transaction.
         * 
         * Use the <code>TransactionContext.getTxId().getSession()</code> methods to access the Session. The Session can
         * be used to access ObjectMaps and modify data in the current transaction.
         * <p>
         * 
         * 
         * @param ctx
         *            the context of the transaction.
         * 
         * @throws TransactionCallbackException
         *             if an error occurs during processing. Any exception will roll back the transaction and will be
         *             included in the TransactionException thrown to the caller.
         * 
         * @see Session#commit()
         * @see TxID
         */
        public void beforeCommit(TransactionContext ctx) throws TransactionCallbackException;

        /**
         * The TransactionContext identifies various information that's available to the beforeCommit() method.
         * 
         * @since 7.1.1
         */
        public interface TransactionContext {
            /**
             * Retrieve the TxID for the transaction.
             * 
             * @return the TxID for the transaction.
             */
            public TxID getTxID();

            /**
             * The LogSequences that reflect the pending changes to the map.
             * 
             * @return the LogSequences.
             */
            public Collection<LogSequence> getLogSequences();
        }
    }
    
    /**
     * Invoked when an <code>ObjectGrid</code> is initialized.
     * <p>
     * This method is called so this object can do any implementation
     * specific intialization.
     *
     * @param objectGrid A reference to the ObjectGrid.
     *
     * @throws TransactionCallbackException if an error occurs during processing
     *
     * @see ObjectGrid#reserveSlot(String)
     */
    void initialize(ObjectGrid objectGrid)
        throws TransactionCallbackException;

    /**
     * Invoked when starting a Session transaction.
     * <p>
     * A TransactionCallback can communicate the begin processing (along with
     * the TxID) to the appropriate BackingMap and/or Loader.  The Loader may
     * use this signal to start a corresponding transaction on the underlying
     * connection to a database.
     *
     * @param id transaction identifer (TxID)
     *
     * @throws TransactionCallbackException if an error occurs during processing
     *
     * @see Session#begin()
     * @see Session#beginNoWriteThrough()
     * @see TxID
     */
    void begin(TxID id)
        throws TransactionCallbackException;

    /**
     * Invoked when committing a Session transaction.
     * <p>
     * This method should be used to commit any underlying transaction and
     * return any underlying connection back to the pool.  The TxID is provided
     * to determine which transaction is being committed
     *
     * @param id transaction identifier (TxID)
     *
     * @throws TransactionCallbackException if an error occurs during processing
     *
     * @see #begin(TxID)
     * @see Session#commit()
     * @see TxID
     */
    void commit(TxID id)
        throws TransactionCallbackException;

    /**
     * Invoked when rolling back a Session transaction.
     * <p>
     * This method should be used to roll back any underlying transaction and
     * return any underlying connection back to the pool.  The TxID is provided
     * to determine which transaction is being committed
     *
     * @param id transaction identifier (TxID)
     *
     * @throws TransactionCallbackException if an error occurs during processing
     *
     * @see #begin(TxID)
     * @see Session#rollback()
     * @see TxID
     */
    void rollback(TxID id)
        throws TransactionCallbackException;

    /**
     * Called when an application attempts to use a Session with no transaction
     * active.
     * <p>
     * The callback could return <code>true</code> in which case an auto
     * <code>Session.begin()</code> is executed. If <code>false</code> is
     * returned, an application exception is thrown indicating no transaction
     * is active. This event is usually used when integrating with a J2EE
     * environment such as WebSphere Application Server.
     *
     * @param session the session which the application is using
     *
     * @return <code>true</code> if an auto begin should be done,
     *         <code>false</code> if this is not the case
     *
     * @see Session
     */
    boolean isExternalTransactionActive(Session session);
}
