// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007,2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ObjectGridSpringFactory.java
//
// Source File Description:
//
//      See javadoc for this class.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b9449        6.1       08/24/2007   bnewport  new code drop
// b10015       6.1       10/01/2007   cheng1    use thread context classloader to load classes
// b10090       XD 6.1    10/10/2007   cdjohnson Fix javadoc
// b10498       XD6.1     11/06/2007   cheng1    Hide Spring messages when no spring xml in classpath
// b11228       XD6.1     02/08/2008   cheng1    Tr.error() in getBeanFactoryForObjectGrid() method
// b21871       XS7.1.1   02/20/2011   jian.tang add InternalBeanFactory
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.spring;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.ObjectGridRuntimeException;
import com.ibm.websphere.objectgrid.plugins.BeanFactory;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.ObjectGridImpl;
import com.ibm.ws.objectgrid.spring.ObjectGridPlatformTransactionManager;
import com.ibm.ws.objectgrid.spring.ScopedOGBeanFactory;
import com.ibm.ws.objectgrid.spring.ShardScope;
import com.ibm.ws.xs.NLSConstants;
import com.ibm.ws.xs.util.Messages;
import com.ibm.ws.xs.util.dopriv.DoPrivUtil;

/**
 * This class serves as a factory to construct instances of the various Spring specific APIs.
 *
 * @since WAS XD 6.1 FIX3, XC10
 */
public final class ObjectGridSpringFactory {
    private static final TraceComponent tc = Tr.register(ObjectGridSpringFactory.class, Constants.TR_GROUP_NAME,
            Constants.TR_RESOURCE_BUNDLE_NAME);

    private static Map/* <String, BeanFactory> */beanFactories = new HashMap();

    /**
     * Scope identifier for shard scope: "shard".
     *
     * @see #getBeanInShardScope(ObjectGrid, String)
     */
    public static final String SCOPE_SHARD = "shard";

    /**
     * This returns an ObjectGrid PlatformLocalTransactionManager.
     *
     * @return the PlatformLocalTransactionManager instance.
     */
    public static SpringLocalTxManager getLocalPlatformTransactionManager() {
        //return new ObjectGridPlatformTransactionManager();
        ObjectGridPlatformTransactionManager ogPlatformTxManager = null;
        try {
            Class c = DoPrivUtil.forName("com.ibm.ws.objectgrid.spring.ObjectGridPlatformTransactionManager", true, ObjectGridSpringFactory.class.getClassLoader());
            Class[] typeParams = new Class[] {};
            Constructor ct = c.getConstructor(typeParams);
            Object[] ct_args = new Object[] {};
            ogPlatformTxManager = (ObjectGridPlatformTransactionManager) ct.newInstance(ct_args);
        } catch (Throwable e) {
            if (tc.isEventEnabled()) {
                Tr.event(tc, "ObjectGridPlatformTransactionManager could not be initialized", e);
            }
        }
        return ogPlatformTxManager;
    }

    /**
     * This returns an adapter for a Spring based bean factory. We use an Object type here to avoid making ObjectGrid
     * dependent on Spring classes being present. A ClassCastException exception is thrown if the supplied factory isn't a
     * Spring BeanFactory instance.
     *
     * @param objectGridName the name of the ObjectGrid
     * @param springBeanFactory
     *            A Spring BeanFactory instance.
     *
     * @throws ClassCastException thrown when the Object type is not a BeanFactory instance.
     */
    public static synchronized void registerSpringBeanFactoryAdapter(String objectGridName, Object springBeanFactory) throws ClassCastException {
        Class c;
		try {
			c = DoPrivUtil.forName("com.ibm.ws.objectgrid.spring.SpringBeanFactory", true, ObjectGridSpringFactory.class.getClassLoader());
            BeanFactory bf = (BeanFactory) c.newInstance();
            Class[] typeparams = new Class[] { Object.class };
            Method m = c.getMethod("setBeanFactory", typeparams);
            Object[] params = new Object[] { springBeanFactory };
            m.invoke(bf, params);
            if(tc.isEventEnabled()) {
                Tr.event(tc, "Registering explicit SpringBeanFactory with grid: " + objectGridName);
            }
            beanFactories.put(objectGridName, bf);
		} catch (Throwable e) {
            FFDCFilter.processException(e, ObjectGridSpringFactory.class.getName()
                    + ".registerSpringBeanFactoryAdapter", "59", ObjectGridSpringFactory.class);
		    if(e instanceof InvocationTargetException) {
		        Throwable cause = e.getCause();
                Tr.error(tc,NLSConstants.FAILED_TO_REGISTER_BEAN_FACTORY_CWOBJ3175E,new Object[]{springBeanFactory.getClass().getName(),objectGridName,cause});
		        if(cause instanceof RuntimeException) {
		            throw (RuntimeException) cause;
		        } else if (cause instanceof Error) {
		            throw (Error) cause;
		        }
		    } else {
		        // This path is unexpected.
                Tr.error(tc,NLSConstants.FAILED_TO_REGISTER_BEAN_FACTORY_CWOBJ3175E,new Object[]{springBeanFactory.getClass().getName(),objectGridName,e});
		        throw new ObjectGridRuntimeException(e);
		    }

		}
    }

    /**
     * This returns the currently registered external bean factory for a named object grid. If no factory has been
     * registered then it attempts to construct a Spring BeanFactory using the xml resource on the class path @ "/X_spring.xml"
     * and /META-INF/X_spring.xml where X is the name of the ObjectGrid. If the xml file is on the class path then the ObjectGrid
     * name MUST be a valid resource name.
     * @param objectGridName
     *            The name of the ObjectGrid
     * @return The BeanFactory instance or null if there were none registered
     */
    public static synchronized BeanFactory getBeanFactoryForObjectGrid(String objectGridName) {
        BeanFactory bf = (BeanFactory) beanFactories.get(objectGridName);
        if (bf == null) {
            try {
                String targetXML = "/" + objectGridName + "_spring.xml";
                URL url = ObjectGridSpringFactory.class.getResource(targetXML);
                if (tc.isEventEnabled()) {
                    Tr.debug(tc, "targetXML = " + targetXML + ", url =" + url);
                }

                // also look in the META-INF directory
                if (url == null) {
                    targetXML = "/META-INF" + targetXML;
                    url = ObjectGridSpringFactory.class.getResource(targetXML);
                }

                if (url != null) {
                    if(tc.isEventEnabled()) {
                        Tr.event(tc, "Registering implicit SpringBeanFactory with grid: " + objectGridName);
                    }


                    Class c = DoPrivUtil.forName("com.ibm.ws.objectgrid.spring.SpringBeanFactory", true, ObjectGridSpringFactory.class.getClassLoader());
                    Class[] typeParams = new Class[] { String.class };
                    Constructor ct = c.getConstructor(typeParams);
                    Object[] ct_args = new Object[] { targetXML };
                    bf = (BeanFactory) ct.newInstance(ct_args);
                    beanFactories.put(objectGridName, bf);
                }
            } catch (Throwable e) {
                // The error only occur when users provide Spring xml file.
                // use Tr.error so it will show up on console.
                FFDCFilter.processException(e, ObjectGridSpringFactory.class.getName()
                        + ".getBeanFactoryForObjectGrid", "139");
                if(e instanceof InvocationTargetException) {
                    // Log a nicer error
                    Throwable cause = e.getCause();
                    Tr.error(tc, Messages.getMsg(NLSConstants.FAILED_TO_GET_BEAN_FACTORY_CWOBJ3176E), new Object[]{objectGridName, cause});
                } else {
                    Tr.error(tc, Messages.getMsg(NLSConstants.FAILED_TO_GET_BEAN_FACTORY_CWOBJ3176E), new Object[]{objectGridName, e});
                }
                bf = null;
            }
        }
        return bf;
    }

    /**
     * This returns an instance of the named Spring bean with the current shard scope using the specified ObjectGrid instance. This
     * allows shard scoped beans to be obtained.
     * @param shard The ObjectGrid instance to use to scope Spring beans using "shard" as scope.
     * @param beanName The bean to return
     * @return The bean instance if it exists
     *
     * @see #SCOPE_SHARD
     */
    static public Object getBeanInShardScope(ObjectGrid shard, String beanName)
    {
    	ObjectGridImpl ogi = (ObjectGridImpl)shard;
    	ShardScope.setScopeBF((ScopedOGBeanFactory)ogi.getSpringBeanFactory());
    	try
    	{
    		return ogi.getSpringBeanFactory().getBean(beanName);
    	}
    	finally
    	{
    		ShardScope.setScopeBF(null);
    	}
    }
}
